﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectMonorail.CollectionGenericObjects;

public class ListGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Список объектов, которые храним
    /// </summary>
    private readonly List<T?> _collection;

    /// <summary>
    /// Максимально допустимое число объектов в списке
    /// </summary>
    private int _maxCount;

    public int Count => _collection.Count;

    public int MaxCount
    {
        get => _maxCount;
        set
        {
            if (value > 0)
            {
                _maxCount = value;
            }
        }
    }

    public CollectionType GetCollectionType => CollectionType.List;

    /// <summary>
    /// Конструктор
    /// </summary>
    public ListGenericObjects()
    {
        _collection = new();
    }

    public T? Get(int position)
    {
        if (position < 0 || position >= Count) return null;
        return _collection[position];
    }

    public int Insert(T? obj)
    {
        if (Count == _maxCount) return -1;
        _collection.Add(obj);
        return Count - 1;
    }

    public int Insert(T? obj, int position)
    {
        if (Count == _maxCount) return -1;
        if (position < 0 || position >= Count) return -1;
        _collection.Insert(position, obj);
        return position;
    }

    public T? Remove(int position)
    {
        if (position < 0 || position >= Count) return null;
        T? obj = _collection[position];
        _collection.RemoveAt(position);
        return obj;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < Count; ++i)
        {
            yield return _collection[i];
        }
    }
}
